" Vim filetype plugin file
" Language:	xslt
" Maintainer:	Johannes Zellner <johannes@zellner.org>
" Last Change:	Fri, 28 Sep 2001 11:06:02 +0200

if exists("b:did_ftplugin") | finish | endif

runtime ftplugin/xml.vim

let b:did_ftplugin = 1
