" VIM syntax file
" Language:	groff
" Maintainer:	Jérôme Plût <Jerome.Plut@ens.fr>
" URL:		http://www.eleves.ens.fr:8080/home/plut/nroff.vim
" Last Change:	2001 Apr 24

" This uses the nroff.vim syntax file.
let b:main_syntax = "nroff"
runtime! syntax/nroff.vim
